<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DataSource\DataSet;

/**
 * Handles Login and Generate Token
 * 
 * Controls login requests, retrieving user data from database and
 * verifying if the user exists. Generates a JWT Token for validation
 *
 * @author Rômulo Alves Lousada
 */
class Login extends \Zeedhi\Framework\Controller\Simple 
{
    /**
     * Timer for Token Expiration
     *
     * Predefined timer to add seconds to token expiration time.
     * Expected values are:
     * 
     * 43200 - 12 Hours
     * 86400 - 1 Day
     * 172800 - 2 Days
     * 604800 - 1 Week
     *
     * @var int
     */
    const TOKEN_TIMER = 86400;

    protected $jwt;
    protected $loginService;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param Util $util Object with utilities functions
     * @param JWT $jwt Object that allows to create and validate tokens
     * @param Login $loginService Object to access LoginService
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct(\Util\JWT $jwt, \Service\Login $loginService) 
    {
        $this->jwt = $jwt;
        $this->loginService = $loginService;
    }
  
    /**
     * Receives and validade information from login
     * 
     * Receives the data sent from the frontend and calls functions from the 
     * providers to search the database for a registry.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function auth(Request $request, Response $response) 
    {
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            if (!isset($input)) {
                throw new \Exception('Não foram recebidos parâmetros');
            }

            $input["user"] = $this->fillUser($input["user"]);
            
            $user = $this->loginService->getUser($input["user"]);
            
            if (sizeof($user)) {
                if ($this->checkPassword($input["password"], $user[0]["CDSENHAOPERWEB"])) {

                    $data = $this->loginService->getOperatorGroupData($user[0]["CDOPERADOR"]);
                    $return = array(
                        'colors'         => $data[0]['IDALTERCORES'],
                        'changeLanguage' => $data[0]['IDALTERIDIOMAS'],
                        'configs'        => $data[0]['IDALTERCONFIG'],
                        'cashierClosing' => $data[0]['IDEMITIRFECHA'],
                        'cancelSale'     => $data[0]['IDCANCELARVENDAS'],
                        'blockProduct'   => $data[0]['IDBLOCKPROD'],
                        'changeMode'     => $data[0]['IDALTERMODAL'],
                        'printer'        => $data[0]['IDALTERIMP'],
                        'layout'         => $data[0]['IDALTERLAYOUT'],
                        'generalConfigs' => $data[0]['IDALTERCONFIGGERAIS'],
                        'images'         => $data[0]['IDALTERIMAGENS'],
                        'importConfigs'  => $data[0]['IDIMPORTARCONFIG'],
                        'exportConfigs'  => $data[0]['IDEXPORTARCONFIG'],
                        'operatorCode'   => $user[0]["CDOPERADOR"],
                        'token'          => $this->generateToken($user[0])
                    );

                    $response->addDataSet(
                        new DataSet('', [$return])
                    );
                    
                } else {
                    $response->addMessage(new Message('Credenciais inválidas.', 'E'));
                }
            } else {
                $response->addMessage(new Message('Credenciais inválidas.', 'E'));
            }
            
        } catch (\Exception $e) {
            $message = 'Erro ao executar o login. '.$e->getMessage();
            $response->addMessage(new Message($message, 'E'));
        }
    }

    /**
     * Fill user string
     * 
     * Check if user string is a numeric string. If it is numeric, pad
     * the string to 12 characters with leading "0"s.
     * 
     * If it's not a numeric string, return it as it is. 
     * 
     * @param String $user Inputted value for user.
     * 
     * @return String
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function fillUser($user) 
    {
        if (is_numeric($user)) {
            return str_pad($user, 12, "0", STR_PAD_LEFT);
        }

        return $user;
    }

    /**
     * Validates password.
     * 
     * Check if inputted password md5 is equal to stored password
     * in database.
     * 
     * @param String $inputPassword Inputted value for password.
     * @param String $databasePassword Retrieved password from database.
     * 
     * @return Boolean
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function checkPassword($inputPassword, $databasePassword) 
    {
        return md5($inputPassword) === $databasePassword;
    }

    /**
     * Token Generator
     * 
     * Generates a Token based on the login data passed and return it.
     *
     * @param Array $user – Array with logged user data.
     *
     * @return string
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function generateToken($user)
    {
        $arrayDataToken = array(
            "id" => $user["CDOPERADOR"],
            "name" => $user["NMOPERADOR"],
            "exp" => time() + self::TOKEN_TIMER
        );

        return $this->jwt->generateToken($arrayDataToken);
    }

    public function validateAccessControl(Request $request, Response $response) {
        try {

            $accessControl = $this->loginService->getAccessControl();

            $return = array(
                'utilityAccessControl' => !empty($accessControl) ? true : false
            );
            $response->addDataSet(new DataSet('', [$return]));
        }  catch (\Exception $e) {
            $message = 'Erro ao executar o login. '.$e->getMessage();
            $response->addMessage(new Message($message, 'E'));
        }
    }

    public function getSubsidiaries(Request $request, Response $response) {

        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);

        try {
            
            if(isset($input['operatorCode']) && !empty($input['operatorCode'])) {
                $cdOperador = $input['operatorCode'];
            } else {
                throw new \Exception("Operador não informado.");
                
            }
            
            $branchs = $this->loginService->getPosPerBranch($cdOperador);

            if(isset($branchs['data']) && empty($branchs['data'])) {
                $response->addMessage(new Message("Não foi encontrado nenhuma filial para o Operador."));
            }

            if($branchs['error']) {
                throw new \Exception($branchs['message']);
                
            }
            
            $return = [];
            $tmpArr = [];

            foreach($branchs['data'] as $branch) {
                
                if((!empty($tmpArr)) && ($tmpArr['subsidiaryCode'] != $branch['CDFILIAL'])) {
                    array_push($return, $tmpArr);
                    $tmpArr = [];
                }

                if(empty($tmpArr)) {
                    $tmpArr = [
                        'subsidiaryCode' => $branch['CDFILIAL'],
                        'subsidiaryName' => $branch['NMFILIAL'],
                        'pos' => []                    
                    ];
                }
                
                $arrPos = [
                    'posCode'      => $branch['CDCAIXA'],
                    'posName'      => $branch['NMCAIXA'],
                    'storeCode'    => $branch['CDLOJA'],
                    'emissionType' => $branch['IDTPEMISSAOFOS'],
                ];
                
                array_push($tmpArr['pos'], $arrPos);

            }

            if(!empty($tmpArr['pos']))
                array_push($return, $tmpArr);
            
            $response->addDataSet(new DataSet('', $return));
        }  catch (\Exception $e) {
            $message = 'Erro na consulta de filiais e caixas. '.$e->getMessage();
            $response->addMessage(new Message($message, 'E'));
        }
    }

}